@@ -11,7 +11,7 @@
 *************************************************************************
 ** Internal interface definitions for SQLite.
 **
-** @(#) $Id: sqliteInt.h,v 1.20 2003/07/31 14:37:22 matt Exp $
+** @(#) $Id: sqliteInt.h,v 1.21 2003/08/23 10:52:51 matt Exp $
 */
 #include "config.h"
 #include "sqlite.h"
@@ -87,6 +87,7 @@
 /* #define SQLITE_OMIT_AUTHORIZATION  1 */
 /* #define SQLITE_OMIT_INMEMORYDB     1 */
 /* #define SQLITE_OMIT_VACUUM         1 */
+/* #define SQLITE_OMIT_TIMEDATE_FUNCS 1 */
 
 /*
 ** Integers of known sizes.  These typedefs might change for architectures
@@ -653,6 +654,7 @@ struct Expr {
 */
 struct ExprList {
   int nExpr;             /* Number of expressions on the list */
+  int nAlloc;            /* Number of entries allocated below */
   struct ExprList_item {
     Expr *pExpr;           /* The list of expressions */
     char *zName;           /* Token associated with this expression */
@@ -679,6 +681,7 @@ struct ExprList {
 */
 struct IdList {
   int nId;         /* Number of identifiers on the list */
+  int nAlloc;      /* Number of entries allocated for a[] below */
   struct IdList_item {
     char *zName;      /* Name of the identifier */
     int idx;          /* Index in some Table.aCol[] of a column named zName */
@@ -697,7 +700,8 @@ struct IdList {
 ** now be identified by a database name, a dot, then the table name: ID.ID.
 */
 struct SrcList {
-  int nSrc;        /* Number of tables or subqueries in the FROM clause */
+  u16 nSrc;        /* Number of tables or subqueries in the FROM clause */
+  u16 nAlloc;      /* Number of entries allocated in a[] below */
   struct SrcList_item {
     char *zDatabase;  /* Name of database holding this table */
     char *zName;      /* Name of the table */
